/* 
MORTGAGE3.CPP:   Compute and display mortgage payment, input values
 
version:         1/22/2006 09:00 
uses:            iostream, iomanip, cmath 
tabs:            every 4 
module type:     exe
*/
 
#include <iostream> 
#include <iomanip> 
#include <cmath>
  
/* 
 * This program computes the monthly payment for a mortgage using amounts for    
 * the principal, interest, and term that are entered by the user.    The formula for    
 * the monthly payment is derived at http://www.hughchou.org/calc/formula.html. It    
 * then displays an amortization schedule for the mortgage.

 * This use of Booleans in this program needs to be improved!
*/
 
using namespace std;
 
#define FOREVER for (;;)
 
/* Global data */
 
const char* ClassName = "Accounting 101" ; 
const int AssignmentNumber = 3; 
     
/* 
 * hello() -- print greeting message 
 *    Just change the class and greeting and it's updated! 
 */ 
void hello(void) { 
    cout << endl << ClassName << " Assignment #" << AssignmentNumber << endl << endl; 
}
 
/* 
 * bye() -- print signoff message 
 */ 
void bye(void) { 
    cout << endl; 
}
 
/* 
 * printDollars(amount, endl) -- format and print amount as dollars with leading $ 
 *    Optional flag passed ends line if set true 
 */ 
void printDollars(double amount, bool endsline=false, bool symbol=true) { 
    if (true == symbol) 
        cout << " $"; 
    else 
        cout.width(11); 
    cout.setf(ios::fixed); 
    cout << setprecision(2) << amount;     
    if (true == endsline) 
        cout << endl; 
} 
     
/* 
 * printPercentage(amount, endl) -- format and print amount as % with trailing sign 
 *    Optional flag passed ends line if set true 
 */ 
void printPercentage(double amount, bool endsline=false) { 
    cout.setf(ios::fixed); 
    cout << setprecision(2) << amount << "%";    
    if (true == endsline) 
        cout << endl;     
} 
     
/* 
 * printInteger(amount, endl) -- format and print integer amount 
 *    Optional flag passed ends line if set true 
 */ 
void printInteger(double amount, bool endsline=false) { 
    cout.width(5); 
    cout.setf(ios::fixed); 
    cout << setprecision(0) << amount;     
    if (true == endsline) 
        cout << endl; 
}
 
/* 
 * printMortgageInfo(p, months, i, m) -- display mortgage information 
 *    Tried to make code look tabular, and use formatting routines. 
 */ 
void printMortgageInfo(double p, double i, double years, double m) { 
    cout << endl; 
    cout << "Principal is ";                printDollars(p, true);
 
    cout << "Term is " ;                        printInteger(years*12); 
    cout << " months or " ;             printInteger(years); 
    cout << " years. " << endl ;
 
    cout << "Interest rate is " ; printPercentage(i/12); 
    cout << " monthly or " ;            printPercentage(i); 
    cout << " annually." << endl ; 
     
    cout << "Monthly payment is ";    printDollars(m, true); 
}
 
/* 
 * computePayment(P, interest, years) -- compute mortgage monthly payment 
 *    Return the monthly payment on a mortgage with principal P, 
 *        interest rate interest, and term years 
 */ 
double computePayment(double P, double interest, double years) { 
    double months = years * 12 ;                // convert loan term to months 
    double i = interest * 0.01 / 12 ;     // convert interest to monthly rate 
     
    double term = pow((1 + i), -months) ; 
    double m = P * (i    / (1-term)) ; 
    return m; 
} 
     
/* 
 * remainingPrincipal(P, interest, years, t) -- compute and returns remaining principal 
 *    P is the original principal amount 
 *    i is the interest rate 
 *    years is the term of the loan 
 *    t is the number of monthly payments made already 
 *    B = L[(1 + c)**n - (1 + c)**p]/[(1 + c)**n - 1] from    
 *        http://www.mtgprofessor.com/formulas.htm 
 */ 
double remainingPrincipal(double P, double interest, double years, int payments) { 
    if (payments == 0) 
        return P; 
         
    double N = years * 12 ;                         // convert loan term to months 
    double J = interest * 0.01 / 12 ;     // convert interest to monthly rate 
     
    double term1 = pow((1+J),N); 
    double term2 = pow((1+J),payments); 
    double ratio = (term1 - term2)/(term1 - 1); 
    double remainingP = P * ratio; 
    return remainingP; 
} 
     
/* 
 * scheduleHeader() -- put a nice title on the schedule, every page full lines 
 */ 
void scheduleHeader(bool clearScreen=true) { 
    if (true == clearScreen) 
        system("CLS");  
    cout << "Month   Interest  Principal    Payment    Balance  Total Int." << endl; 
} 
     
/* 
 * printAmortizationSchedule(P, i, years) -- print mortgage amortization schedule 
 *    P is principal 
 *    i is annual interest as X.XX 
 *    years is term of mortgage 
 */ 
void printAmortizationSchedule(double P, double interest, double years) { 
    double monthlyinterest, monthlyprincipal, whatsleft; 
    const int PageSize = 24; // number of months on a "page"
 
    double priorbalance = P; 
    double payment = computePayment(P, interest, years); 
    double totalinterest = 0.0; 
    int months = (int)(years * 12);
 
    getchar(); // extra character to drain 1st 
    scheduleHeader(false); 
    for (int i = 0; i < months; i++) { 
        // compute amounts 
        whatsleft = remainingPrincipal(P, interest, years, i+1); 
        monthlyprincipal = priorbalance - whatsleft; 
        monthlyinterest = payment - monthlyprincipal; 
        totalinterest += monthlyinterest; 
         
        // print one line of schedule 
        printInteger(i+1); 
        printDollars(monthlyinterest, false, false); 
        printDollars(monthlyprincipal, false, false); 
        printDollars(payment, false, false); 
        printDollars(whatsleft, false, false); 
        printDollars(totalinterest, true, false); 
         
        // carry balance forward 
        priorbalance = whatsleft; 
         
        // should we paginate? 
        if (0 == ((i+1) % PageSize) && ((i+1) < months)) { 
            char c = toupper(getchar()); 
            if (c == 'Q') 
                break; 
            scheduleHeader(); 
        } 
    } 
} 
     
/* 
 * inputAmount(prompt) -- enter and return a numeric amount 
 *    Displays prompt message 
 *    Returns amount 
 *    Loops until amount greater than zero is received 
 *    If allowExit is true, then a zero amount will exit program 
 */ 
double inputAmount(char *prompt, bool allowExit=false) {     
    double amount = 0.0; 
    do { 
        cout << prompt << (allowExit ? " (use 0 to exit)->" : "->") ;    
        cin >> amount ; 
        if ((false == allowExit) && (amount <= 0.0)) { 
            cout << endl << "Please enter an amount greater than zero!" << endl; 
        } else if ((true == allowExit) && (amount == 0.0)) { 
            bye(); 
            exit(0); 
        } 
    } while (amount <= 0.0);    
    return amount; 
} 
     
/* 
 * main() -- compute mortgage from hard-coded values 
 */ 
int main(void) { 
    hello(); 
    FOREVER { 
        double P = inputAmount("Enter principal ($)", true); 
        double interest = inputAmount("Enter interest rate (%)"); 
        double years = inputAmount("Enter term in YEARS"); 
        double m = computePayment(P, interest, years); 
        printMortgageInfo(P, interest, years, m) ; 
        cout << endl << endl; 
        printAmortizationSchedule(P, interest, years); 
        cout << endl; 
    } 
    bye();
    return 0;
}
